#include <sdcc-lib.h>
#include <pic16f648a.h>	// /usr/share/sdcc/include/pic/pic16f648a.h
#include "inline.h"

//__code unsigned char at 0x2000 id_location[] = {'A', 'B', 'C', 'D', 0, 0, 0 };

unsigned int at 0x2007 __config =
	_CP_OFF &
	_DATA_CP_OFF &
	_LVP_OFF &
	_BOREN_OFF &	// for 3V
	_MCLRE_OFF &
	_PWRTE_OFF &
	_WDT_OFF &
	_INTRC_OSC_NOCLKOUT;

__code unsigned char at 0x2100 eedata[] = {'d', 'u', 'm', 'm', 'y', 0 };

#define LED_BIT		0

void initSerial();
unsigned char receiveSerial();
void transmitSerial(unsigned char c);

void forCalibOsc();

volatile unsigned char waitCntS;
volatile unsigned char waitCntM;
volatile unsigned char waitCntL;

void waitS();
void waitM();
void waitL();

#define waitMM(x) LD_A(x); waitM();
#define waitLL(x) LD_A(x); waitL();

void Intr(void) __interrupt 0 {  
}

void main(void) {
	unsigned char sa, sb;
	unsigned char recv;

						// 76543210
	PORTA		= 0x00;	// 00000000		RA5: Input only
	CMCON		= 0x07;
	TRISA		= 0x20;	// 00100000		1: 入力, 0: 出力

	PORTB		= 0x00;	// 00000000
	TRISB		= 0x02;	// 00000010		1: 入力, 0: 出力

	OPTION_REG	= 0xFF; // 初期値:11111111b
						// 7:PORTBプルアップ(0:使用する)
						// 6:RB0割り込みポート変化選択(0:立下り)
						// 5:TMR0クロックソース選択ビット(0:内部)
						// 4:TMR0加算タイミング(0:立上り)
						// 3:プリスケーラ割り当てビット(0:TMR0)
						// 2~0プリスケーラレート選択ビット(011:1/16)
						// 4MHz => 1MHz = 1,000,000 / 16 / 256 = 244Hz

	INTCON		= 0x00; // 初期値:0000000xb
						// 7:GIE  Global Interrupt Enable bit
						// 6:PEIE Peripheral Interrupt Enable bit
						// 5:T0IE TMR0 Overflow Interrupt Enable bit
						// 4:INTE RB0/INT External Interrupt Enable bit
						// 3:RBIE RB Port Change Interrupt Enable bit
						// 2:T0IF TMR0 Overflow Interrupt Flag bit
						// 1:INTF RB0/INT External Interrupt Flag bit
						// 0:RBIF RB Port Change Interrupt Flag bit

//	PIE1		= 0x00; // 初期値:00000000b
//	PIR1		= 0x00; // 初期値:00000000b
						// 7:EEIE EE Write Complete Interrupt Enable Bit
						// 6:CMIE Comparator Interrupt Enable bit
						// 5:RCIE USART Receive Interrupt Enable bit
						// 4:TXIE USART Transmit Interrupt Enable bit
						// 2:CCP1IE CCP1 Interrupt Enable bit
						// 1:TMR2IE TMR2 to PR2 Match Interrupt Enable bit
						// 0:TMR1IE TMR1 Overflow Interrupt Enable bit

//	forCalibOsc();

	initSerial();

	while(1) {

		sa = 0;
		sb = 0;

		recv = receiveSerial();

		if(recv & 0x80) {			// リビング南
			if(recv & 0x08) {			// 開 
				sa |= 0x04;
			} else if(recv & 0x04) {	// 止
				sa |= 0x02;
			} else if(recv & 0x02) {	// 閉
				sa |= 0x01;
			}
		}

		if(recv & 0x40) {			// リビング東
			if(recv & 0x08) {			// 開 
				sa |= 0x80;
			} else if(recv & 0x04) {	// 止
				sa |= 0x40;
			} else if(recv & 0x02) {	// 閉
				sb |= 0x80;
			}
		}

		if(recv & 0x20) {			// 和室
			if(recv & 0x08) {			// 開 
				sb |= 0x40;
			} else if(recv & 0x04) {	// 止
				sb |= 0x20;
			} else if(recv & 0x02) {	// 閉
				sb |= 0x10;
			}
		}

		PORTA = sa;
		PORTB = sb;
		waitLL(2);
		PORTA = 0;
		PORTB = 0;

		transmitSerial(recv);
	}
}

/*------------------------------------------------------------------------------
 *
 *	シリアル初期設定
 *
 */
void initSerial() {
									// Baud Rate = FOSC / (16 * (X + 1))
									// 9615.38 = 4,000,000 / (16 * (25 + 1))

	SPBRG	= 25;					// Baud Rate Generator
	BRGH	= 1;					// High Baud Rate Select(High/Low)
	SYNC	= 0;					// USART Mode Select(Synchronous/Asynchronous)

	SPEN	= 1;					// Serial Port Enable
	CREN	= 1;					// Continuous Receive Enable

	TXEN	= 1;					// Transmit Enable
}

/*------------------------------------------------------------------------------
 *
 *	シリアル受信処理
 *		データ受信までブロックされる
 *		受信データをAレジスタに入れて返る
 *
 */
unsigned char receiveSerial() {

	unsigned char recv;

	while(1) {
		while(!RCIF);							// 受信フラグチェック

		if(OERR) {								// オーバランエラーチェック
			CREN = 0; CREN = 1;					// 受信回路リセット(OERRクリア)
		}
		if(FERR) {								// フレーミングエラーチェック
			recv = RCREG;						// ゴミを吸い取る
			continue;
		}
		break;
	}
	recv = RCREG;								// エラーなし、受信データを得る 
	return(recv);
}

unsigned char receiveSerial2() {				// ノンブロック

	unsigned char recv;

	if(!RCIF)									// 受信フラグチェック
		return(0x39);

	while(1) {
		if(OERR) {								// オーバランエラーチェック
			CREN = 0; CREN = 1;					// 受信回路リセット(OERRクリア)
		}
		if(FERR) {								// フレーミングエラーチェック
			recv = RCREG;						// ゴミを吸い取る
			return(0x38);
		}
		break;
	}
	recv = RCREG;								// エラーなし、受信データを得る 
	return(recv);
}
/*
		BIT	OERR, (RCSTA)		; オーバランエラーチェック
		JP	NZ, ovRunErr
		BIT	FERR, (RCSTA)		; フレーミングエラーチェック
		JP	NZ, frameErr
		LD	A, (RCREG)		; エラーなし、受信データを得る
		RET
ovRunErr
		RES	CREN, (RCSTA)
		SET	CREN, (RCSTA)		; 受信回路リセット(OERRクリア)
		LD	A, 0E1h			; '○'
		BIT	FERR, (RCSTA)		; フレーミングエラーチェック
		RET	Z
frameErr
		LD	A, (RCREG)		; ゴミを吸い取る
		LD	A, 87h			; '□'
		RET
*/

void transmitSerial(unsigned char c) {

		TXREG = c;								// USART Transmit data register
}

/*------------------------------------------------------------------------------
 *
 *	20秒周期LED点滅処理(クロック速度計測用)
 *
 */
void forCalibOsc() {
	_asm
forCalibOsc:					;			forCalibOsc:
		bsf	PORTB, LED_BIT		;							SET		LED_BIT, (PORTB)
						;	
		movlw	100			;							LD		A, 100					; 100を指定すると、
		call	WAITnx01sm25u		;							CALL	WAITnx01sm25u			; 2行で 100 x 100ms - 25us 消費する
		movlw	6			;							LD		A, 6					; 10を指定すると、
		call	WAITnx3p5u		;							CALL	WAITnx3p5u				; 2行で 6 x 3 + 5 = 23us 消費
						;	
		bcf	PORTB, LED_BIT		;							RES		LED_BIT, (PORTB)
						;	
		movlw	100			;							LD		A, 100					; 100を指定すると、
		call	WAITnx01sm25u		;							CALL	WAITnx01sm25u			; 2行で 100 x 100ms - 25us 消費する
		movlw	6			;							LD		A, 6					; 10を指定すると、
		call	WAITnx3p5u		;							CALL	WAITnx3p5u				; 2行で 6 x 3 + 5 = 23us 消費
						;	
		goto	forCalibOsc		;							JP		forCalibOsc
	_endasm;
}

void waitS() {
	_asm
						;			;-------------------------------------------------------------------------------
						;			;
						;			;	時間待ち(小)
						;			;
						;			;	    * 4MHz 時
						;			;				LD		A, 10					; 10を指定すると、
						;			;				CALL	WAITnx3p5u				; 2行で 10 x 3 + 5 = 35us 消費
						;			;
WAITnx3p5u:					;			WAITnx3p5u:
		movwf	_waitCntS		;							LD		(_waitCntS), A
WAITnx3p5u1:					;			WAITnx3p5u1:
		decfsz	_waitCntS		
		goto	WAITnx3p5u1		;							DJNZ	(_waitCntS), WAITnx3p5u1
						;			;				RET
	_endasm;
}

void waitM() {
	_asm
						;			;-------------------------------------------------------------------------------
						;			;
						;			;	時間待ち(中)
						;			;
						;			;	    * 4MHz 時
						;			;				LD		A, 20					; 0.01秒が欲しい場合、20を指定すると、
						;			;				CALL	WAITnx500m35u			; 2行で 20 x 500 - 35 = 9.965ms 消費する
						;			;
WAITnx500m35u:					;			WAITnx500m35u:
		movwf	_waitCntM		;							LD		(_waitCntM), A
		goto	WAITnx500m35u2		;							JP		WAITnx500m35u2
WAITnx500m35u1:	nop				;			WAITnx500m35u1:	NOP								;             =   1us
		movlw	12			;							LD		A, 12					;
		call	WAITnx3p5u		;							CALL	WAITnx3p5u				;  12 x 3 + 5 =  41us
WAITnx500m35u2:	movlw	150			;			WAITnx500m35u2:	LD		A, 150					;
		call	WAITnx3p5u		;							CALL	WAITnx3p5u				; 150 x 3 + 5 = 455us
		decfsz	_waitCntM		
		goto	WAITnx500m35u1		;							DJNZ	(_waitCntM), WAITnx500m35u1
						;			;				RET
	_endasm;
}

void waitL() {
	_asm
						;			;-------------------------------------------------------------------------------
						;			;
						;			;	時間待ち(大)
						;			;
						;			;	    * 4MHz 時
						;			;				LD		A, 10					; 1秒が欲しい場合、10を指定すると、
						;			;				CALL	WAITnx01sm25u			; 2行で 10 x 100ms - 25us 消費する
						;			;
WAITnx01sm25u:					;			WAITnx01sm25u:
		movwf	_waitCntL		;							LD		(_waitCntL), A
		goto	WAITnx01sm25u2		;							JP		WAITnx01sm25u2
WAITnx01sm25u1:	movlw	9			;			WAITnx01sm25u1:	LD		A, 9					;
		call	WAITnx3p5u		;							CALL	WAITnx3p5u				;   9 x   3 +  5 =    32us
WAITnx01sm25u2:	movlw	200			;			WAITnx01sm25u2:	LD		A, 200					;
		call	WAITnx500m35u		;							CALL	WAITnx500m35u			; 200 x 500 - 35 = 99965us
		decfsz	_waitCntL		
		goto	WAITnx01sm25u1		;							DJNZ	(_waitCntL), WAITnx01sm25u1
						;			;				RET
	_endasm;
}

